@@ -1,3 +1,4 @@
+use crate::graphics::GraphicsExt;
 /**
  * Copyright 2019-2020, Benjamin Vaisvil and the zenith contributors
  */
@@ -41,6 +42,9 @@ use tui::widgets::{
 use tui::Frame;
 use unicode_width::UnicodeWidthStr;
 
+#[cfg(all(target_os = "linux", feature = "nvidia"))]
+use nvml::error::NvmlError;
+
 const PROCESS_SELECTION_GRACE: Duration = Duration::from_millis(2000);
 const LEFT_PANE_WIDTH: u16 = 34u16;
 
@@ -318,11 +322,11 @@ fn render_process_table(
                 "{:>5.1}",
                 p.get_io_wait(&app.histogram_map.tick)
             )));
+            #[cfg(feature = "nvidia")]
+            row.push(Cell::from(format!("{:>4.0}", p.gpu_usage)));
+            #[cfg(feature = "nvidia")]
+            row.push(Cell::from(format!("{:>4.0}", p.fb_utilization)));
 
-            if !app.gfx_devices.is_empty() {
-                row.push(Cell::from(format!("{:>4.0}", p.gpu_usage)));
-                row.push(Cell::from(format!("{:>4.0}", p.fb_utilization)));
-            }
             row.push(Cell::from(format!("{:}{:}", p.name, cmd_string)));
 
             let row = Row::new(row);
@@ -355,11 +359,10 @@ fn render_process_table(
     ];
     #[cfg(target_os = "linux")]
     header.push(String::from("IOWAIT% "));
-
-    if !app.gfx_devices.is_empty() {
-        header.push(String::from("GPU% "));
-        header.push(String::from("FB%  "));
-    }
+    #[cfg(feature = "nvidia")]
+    header.push(String::from("GPU% "));
+    #[cfg(feature = "nvidia")]
+    header.push(String::from("FB%  "));
     //figure column widths
     let mut widths = Vec::with_capacity(header.len() + 1);
     let mut used_width = 0;
@@ -1570,7 +1573,12 @@ fn render_section_mgr(list: &mut SectionMGRList<'_>, area: Rect, f: &mut Frame<'
     f.render_stateful_widget(list_widget, layout[1], &mut list.state);
 }
 
-fn render_help(area: Rect, f: &mut Frame<'_, ZBackend>, history_recording: HistoryRecording) {
+fn render_help(
+    _app: &CPUTimeApp,
+    area: Rect,
+    f: &mut Frame<'_, ZBackend>,
+    history_recording: HistoryRecording,
+) {
     let header_style = Style::default().fg(Color::Green);
     let main_style = Style::default();
     let key_style = main_style.fg(Color::Cyan);
@@ -1650,6 +1658,23 @@ fn render_help(area: Rect, f: &mut Frame<'_, ZBackend>, history_recording: Histo
         }
     }
 
+    #[cfg(all(target_os = "linux", feature = "nvidia"))]
+    if let Some(ne) = &_app.nvml_error {
+        let content = match ne {
+            NvmlError::DriverNotLoaded => "NVIDIA Error: No Driver Detected.",
+            NvmlError::NoPermission => "NVIDIA Error: Permissioned denied to talk to driver.",
+            NvmlError::Unknown => "NVIDIA Error: Unkown Error.",
+            _ => "",
+        };
+        if content.len() > 0 {
+            t.push(Spans::from(vec![Span::styled("", header_style)]));
+            t.push(Spans::from(vec![Span::styled(
+                content,
+                Style::default().fg(Color::Yellow),
+            )]));
+        }
+    }
+
     let help_height = t.len() as u16;
 
     let help_layout = Layout::default()
@@ -1808,7 +1833,7 @@ impl<'a> TerminalRenderer<'_> {
         disable_history: bool,
     ) -> TerminalRenderer {
         debug!("Create Metrics App");
-        let app = CPUTimeApp::new(Duration::from_millis(tick_rate), db_path);
+        let mut app = CPUTimeApp::new(Duration::from_millis(tick_rate), db_path);
         debug!("Create Event Loop");
         let events = Events::new(app.histogram_map.tick);
 
@@ -1821,7 +1846,11 @@ impl<'a> TerminalRenderer<'_> {
         terminal.hide_cursor().ok();
 
         let constraints = get_constraints(section_geometry, terminal_size().1);
-        let section_geometry = section_geometry.to_vec();
+        let mut section_geometry = section_geometry.to_vec();
+        app.update_gfx_devices();
+        if app.gfx_devices.len() == 0 {
+            section_geometry.retain(|(section, _)| *section != Section::Graphics);
+        }
         TerminalRenderer {
             terminal,
             app,
@@ -1942,7 +1971,7 @@ impl<'a> TerminalRenderer<'_> {
                             (false, true) => HistoryRecording::UserDisabled,
                             (false, false) => HistoryRecording::OtherInstancePrevents,
                         };
-                        render_help(v_sections[1], f, history_recording);
+                        render_help(app, v_sections[1], f, history_recording);
                     } else if show_section_mgr {
                         let v_sections = Layout::default()
                             .direction(Direction::Vertical)
